#
#    Uncomplicated VM Builder
#    Copyright (C) 2007-2009 Canonical Ltd.
#    
#    See AUTHORS for list of contributors
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3, as
#    published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
import unittest

from VMBuilder.util      import checkroot, run_cmd
from VMBuilder.exception import VMBuilderException, VMBuilderUserError

class TestUtils(unittest.TestCase):
    def test_run_cmd_accept_environment(self):
        "run_cmd sets environment variables passed to it"

        self.assertTrue("foobarbaztest=bar" in run_cmd("env", env={'foobarbaztest' : 'bar' }))

    def test_run_cmd_discards_stderr(self):
        "Stuff sent to stderr is not returned by run_cmd"

        self.assertEqual(run_cmd('cp', '/dev/stdin', '/dev/stderr', stdin='this comes from stdin'), '')

    def test_run_cmd_returns_stdin(self):
        "Stuff sent to stdin is returned by run_cmd"

        self.assertEqual(run_cmd('cp', '/dev/stdin', '/dev/stdout', stdin='this comes from stdin'), 'this comes from stdin')

    def test_run_cmd_raises_exception_on_non_zero_return(self):
        "If command has non-zero exit code, run_cmd raises VMBuilderException"

        self.assertRaises(VMBuilderException, run_cmd, '/bin/false')

    def test_run_cmd_locale(self):
        "run_cmd sets LANGand LC_ALL to C"

        self.assertTrue("LANG=C" in run_cmd("env"))
        self.assertTrue("LC_ALL=C" in run_cmd("env"))

    def test_checkroot(self):
        "checkroot raises an error if euid is not 0, and otherwise doesn't"
        geteuid_saved = os.geteuid

        os.geteuid = lambda : 0
        self.assertEqual(checkroot(), None)

        os.geteuid = lambda : 1000
        self.assertRaises(VMBuilderUserError, checkroot)

        os.geteuid = geteuid_saved
